/*--------------------------------------------------------------------------------------------+
|                                                                                             | 
|      AT89C51 as a decoder of RF modules AM-HRR3-433, AM Supter Regenerative Receivers       |
|                                                                                             |
| Name : reception.c                                                                          |
| Purpose:                                                                                    |
|       For many remote applications, encoder/decoder pair IC always used for simpler design. |
|       This program provide the function same as RF decoder IC. Saving the cost for buying   | 
|       the encoder and decoder pair IC. Also, the transmission capacity, data rate should be |
|       higher, as we direct access the RF modules pair.                                      |
| 		Get the rf series byte data through the air and then export it to P1.                 |
|       The program is use the polling technique. Data protocol is asynchronous mode, that is |
|       1 start bit + 8 bits byte data + 2 stop bit, total 11 bits.                           |
|                                                                                             |
| written by: Dillian Wong 27/5/04                                                            |
+--------------------------------------------------------------------------------------------*/

/*____________ I N C L U D E S _________________________________________________________________*/
#include <reg51.h>
#include <stdio.h>
#include "LCDDriver\LCDConfig.h"
#include "rPrintf\rprintf.h"

/*____________ HARDWARE PIN CONNECTION_______________________________________________________*/
// define hardware pin connection 
sbit DIN = P3^3;

/*____________ P R O T O C O L ______________________________________________________________*/
char getrcvdata(void);
void delay(void);
void delay2(void);

/*____________ C O M  P O R T __________________________________________________________________*/
// com port with 9600 baud with crystal 11.0592MHz.
void init_uart(void){	SCON=0x50; TMOD|=0x20; TH1=253; TR1=1; TI=1; }

/*____________ M A I N   P R O G R A M__________________________________________________________*/

void main(void)
{
	char rcvdata;
	init_uart(); 									// 9600 baud @ 11.0592MHz
	turnOnLCD();
	turnOnRS232();
  	printf("#Rx:\n");

	while(1)
	{
		rcvdata=getrcvdata();
	//	printf("(%c)",rcvdata);
		printf("(0x");
		printHex8(rcvdata);
	//	printf(")");
		printf(")  ");
		P1=rcvdata;
	}
}

/*____________ A S Y C H R O N O U S   R E C E P T I O N _______________________________________*/
/*
	using RF modules is different from direct line connection. The idle state in many design is 
	logic high. However, because RF receiver modules cannot stay in logic high in a long time. 
	Therefore, the idle states in RF is always logic low. In order to stick on the standard 
	asychronous transmission mode, i remain the idle states in logic high. To achieve this, 
	before the data transmit, the transmitter will send a logic high first, to create a virtual 
	idle state. when DOUT falling, that means it is the start bit. and followed by 8 bits data and 
	2 stop bits.
*/
char getrcvdata(void)
{
	bit a,b,c,d,e,f,g,h,i,j,k,l;						//bit data
//	int p;												//checking use!
	char rcvdata=0x00;

    /* START receptions*/
	while(1)											//loop until a valid rcvdata
	{
		while(!DIN)										//rising edge, in idle
		{                                               //trigger + pretend idle states
			delay2();
			if(DIN) {a=1;}else {a=0;}					/*start bit*/
			if(DIN)										//high means a valid start bit
			{		
				delay();
				if(DIN) {b=1;}else {b=0;} delay();		//8 bits byte data
				if(DIN) {c=1;}else {c=0;} delay();
				if(DIN) {d=1;}else {d=0;} delay();
				if(DIN) {e=1;}else {e=0;} delay();
				if(DIN) {f=1;}else {f=0;} delay();
				if(DIN) {g=1;}else {g=0;} delay();
				if(DIN) {h=1;}else {h=0;} delay();
				if(DIN) {i=1;}else {i=0;} delay();
				if(DIN) {j=1;}else {j=0;} delay();		//2 stop bits
				if(DIN) {k=1;}else {k=0;} delay();
				if(DIN) {l=1;}else {l=0;} delay();		//virtual idle states, after the DOUT 
				                                        //will return to 0

		//checking use!
		/*		p=a;printf("%d",p);
				p=b;printf("%d",p);p=c;printf("%d",p);p=d;printf("%d",p);p=e;printf("%d",p);
				p=f;printf("%d",p);p=g;printf("%d",p);p=h;printf("%d",p);p=i;printf("%d",p);
				p=j;printf("%d",p);p=k;printf("%d",p);p=l;printf("%d",p);
		*/
	
				/*extract the byte data */
				if (b) { rcvdata |= 0x01; }		//(LSB)
				if (c) { rcvdata |= 0x02; }
				if (d) { rcvdata |= 0x04; }						
				if (e) { rcvdata |= 0x08; }	 					
				if (f) { rcvdata |= 0x10; }
				if (g) { rcvdata |= 0x20; }
				if (h) { rcvdata |= 0x40; }
				if (i) { rcvdata |= 0x80; }   	//(MSB)
				while(DIN);								//wait until return idle state

				//1 high start bits , 2 low stop bits, 
				//followed by a high bits means return to idle state
				if(a==1 && j==0 && k==0 && l==1)
					return rcvdata;						//return a valid data
				else
					rcvdata=0x00;						//if not valid, clear the buffer
			}
		}//end of while(!DIN)
	}
}


/*____________ DELAY  USE  TIMER0 _______________________________________________________________*/
/*
	delay about 4.34ms
*/
void delay(void)
{
	unsigned int count=1000;
	TR0=0;					//turn off timer0, before setting
	TMOD |=	0x01;			//set time0 as mode0
	TH0=(65536-count)/256; 	//count 2304 machine cycle
	TL0=(65536-count)%256; 	//1 machine cycle = 12/11.0598M = 1.085us
	IE |=0x82;				//2304x1.085u=2.5ms
	TR0=1;					//turn on timer0;
	while(TR0);				//wait until timer0 is off.
}

/*
	delay about 2.17ms
*/
void delay2(void)
{
	unsigned int count=500;
	TR0=0;					//turn off timer0, before setting
	TMOD |=	0x01;			//set time0 as mode0
	TH0=(65536-count)/256; 	//count 2304 machine cycle
	TL0=(65536-count)%256; 	//1 machine cycle = 12/11.0598M = 1.085us
	IE |=0x82;				//2304x1.085u=2.5ms
	TR0=1;					//turn on timer0;
	while(TR0);				//wait until timer0 is off.
}

void timer0_ISR (void) interrupt 1{TR0=0;}
